@@ -44,7 +44,7 @@ def cmd_am_common_opts(sp):
     sp.add_argument('-S', '--sloppy-trailers', dest='sloppytrailers', action='store_true', default=False,
                     help='Apply trailers without email address match checking')
     sp.add_argument('-T', '--no-add-trailers', dest='noaddtrailers', action='store_true', default=False,
-                    help='Do not add or sort any trailers')
+                    help='Do not add any trailers from follow-up messages')
     sp.add_argument('-s', '--add-my-sob', dest='addmysob', action='store_true', default=False,
                     help='Add your own signed-off-by to every patch')
     sp.add_argument('-l', '--add-link', dest='addlink', action='store_true', default=False,
@@ -71,6 +71,21 @@ def cmd_kr(cmdargs):
     b4.kr.main(cmdargs)
 
 
+def cmd_prep(cmdargs):
+    import b4.ez
+    b4.ez.cmd_prep(cmdargs)
+
+
+def cmd_trailers(cmdargs):
+    import b4.ez
+    b4.ez.cmd_trailers(cmdargs)
+
+
+def cmd_send(cmdargs):
+    import b4.ez
+    b4.ez.cmd_send(cmdargs)
+
+
 def cmd_am(cmdargs):
     import b4.mbox
     b4.mbox.main(cmdargs)
@@ -81,15 +96,6 @@ def cmd_shazam(cmdargs):
     b4.mbox.main(cmdargs)
 
 
-def cmd_attest(cmdargs):
-    import b4.attest
-    if len(cmdargs.patchfile):
-        b4.attest.attest_patches(cmdargs)
-    else:
-        logger.critical('ERROR: missing patches to attest')
-        sys.exit(1)
-
-
 def cmd_pr(cmdargs):
     import b4.pr
     b4.pr.main(cmdargs)
@@ -109,7 +115,8 @@ def cmd():
     # noinspection PyTypeChecker
     parser = argparse.ArgumentParser(
         prog='b4',
-        description='A tool to work with public-inbox patches',
+        description='A tool to work with patches in public-inbox archives',
+        epilog='Online docs available at https://b4.docs.kernel.org',
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
     )
     parser.add_argument('--version', action='version', version=b4.__VERSION__)
@@ -164,19 +171,6 @@ def cmd():
                              '(default: 3 weeks)'))
     sp_sh.set_defaults(func=cmd_shazam)
 
-    # b4 attest
-    sp_att = subparsers.add_parser('attest', help='(DEPRECATED) Create cryptographic attestation for a set of patches')
-    sp_att.add_argument('-f', '--from', dest='sender', default=None,
-                        help='OBSOLETE: this option does nothing and will be removed')
-    sp_att.add_argument('-n', '--no-submit', dest='nosubmit', action='store_true', default=False,
-                        help='OBSOLETE: this option does nothing and will be removed')
-    sp_att.add_argument('-o', '--output', default=None,
-                        help='OBSOLETE: this option does nothing and will be removed')
-    sp_att.add_argument('-m', '--mutt-filter', default=None,
-                        help='OBSOLETE: this option does nothing and will be removed')
-    sp_att.add_argument('patchfile', nargs='*', help='Patches to attest')
-    sp_att.set_defaults(func=cmd_attest)
-
     # b4 pr
     sp_pr = subparsers.add_parser('pr', help='Fetch a pull request found in a message ID')
     sp_pr.add_argument('-g', '--gitdir', default=None,
@@ -224,6 +218,8 @@ def cmd():
                        help='Send email instead of writing out .thanks files')
     sp_ty.add_argument('--dry-run', action='store_true', dest='dryrun', default=False,
                        help='Print out emails instead of sending them')
+    sp_ty.add_argument('--pw-set-state', default=None,
+                       help='Set this patchwork state instead of default (use with -a, -t or -d)')
     sp_ty.set_defaults(func=cmd_ty)
 
     # b4 diff
@@ -255,6 +251,72 @@ def cmd():
                        help='Show all developer keys found in a thread')
     sp_kr.set_defaults(func=cmd_kr)
 
+    # b4 prep
+    sp_prep = subparsers.add_parser('prep', help='Work on patch series to submit for mailing list review')
+    spp_g = sp_prep.add_mutually_exclusive_group()
+    spp_g.add_argument('-c', '--auto-to-cc', action='store_true', default=False,
+                       help='Automatically populate cover letter trailers with To and Cc addresses')
+    spp_g.add_argument('-p', '--format-patch', metavar='OUTPUT_DIR',
+                       help='Output prep-tracked commits as patches')
+    spp_g.add_argument('--edit-cover', action='store_true', default=False,
+                       help='Edit the cover letter in your defined $EDITOR (or core.editor)')
+    spp_g.add_argument('--show-revision', action='store_true', default=False,
+                       help='Show current series revision number')
+    spp_g.add_argument('--force-revision', metavar='N', type=int,
+                       help='Force revision to be this number instead')
+    spp_g.add_argument('--manual-reroll', dest='reroll', default=None, metavar='COVER_MSGID',
+                       help='Mark current revision as sent and reroll (requires cover letter msgid)')
+    ag_prepn = sp_prep.add_argument_group('Create new branch', 'Create a new branch for working on patch series')
+    ag_prepn.add_argument('-n', '--new', dest='new_series_name',
+                          help='Create a new branch for working on a patch series')
+    ag_prepn.add_argument('-f', '--fork-point', dest='fork_point',
+                          help='When creating a new branch, use this fork point instead of HEAD')
+    ag_prepn.add_argument('-F', '--from-thread', metavar='MSGID', dest='msgid',
+                          help='When creating a new branch, use this thread')
+    ag_prepe = sp_prep.add_argument_group('Enroll existing branch', 'Enroll existing branch for prep work')
+    ag_prepe.add_argument('-e', '--enroll', dest='enroll_base',
+                          help='Enroll current branch, using the passed tag, branch, or commit as fork base')
+    sp_prep.set_defaults(func=cmd_prep)
+
+    # b4 trailers
+    sp_trl = subparsers.add_parser('trailers', help='Operate on trailers received for mailing list reviews')
+    sp_trl.add_argument('-u', '--update', action='store_true', default=False,
+                        help='Update branch commits with latest received trailers')
+    sp_trl.add_argument('-S', '--sloppy-trailers', dest='sloppytrailers', action='store_true', default=False,
+                        help='Apply trailers without email address match checking')
+    sp_trl.add_argument('-F', '--trailers-from', dest='msgid',
+                        help='Look for trailers in the thread with this msgid instead of using the series change-id')
+    sp_trl.add_argument('--since', default='1.month',
+                        help='The --since option to use with -F when auto-matching patches (default=1.month)')
+    sp_trl.set_defaults(func=cmd_trailers)
+
+    # b4 send
+    sp_send = subparsers.add_parser('send', help='Submit your work for review on the mailing lists')
+    sp_send.add_argument('-d', '--dry-run', dest='dryrun', action='store_true', default=False,
+                         help='Do not send, just dump out raw smtp messages to the stdout')
+    sp_send.add_argument('-o', '--output-dir',
+                         help='Do not send, write raw messages to this directory (forces --dry-run)')
+    sp_send.add_argument('--prefixes', nargs='+',
+                         help='Prefixes to add to PATCH (e.g. RFC, WIP)')
+    sp_send.add_argument('--no-trailer-to-cc', action='store_true', default=False,
+                         help='Do not add any addresses found in the cover or patch trailers to To: or Cc:')
+    sp_send.add_argument('--hide-cover-to-cc', action='store_true', default=False,
+                         help='Hide To: and Cc: entries from the cover letter trailers (but still send to them)')
+    sp_send.add_argument('--to', nargs='+', help='Addresses to add to the To: list')
+    sp_send.add_argument('--cc', nargs='+', help='Addresses to add to the Cc: list')
+    sp_send.add_argument('--not-me-too', action='store_true', default=False,
+                         help='Remove yourself from the To: or Cc: list')
+    sp_send.add_argument('--resend', default=None,
+                         help='Resend a previously sent version of the series')
+    sp_send.add_argument('--no-sign', action='store_true', default=False,
+                         help='Do not add the cryptographic attestation signature header')
+    ag_sendh = sp_send.add_argument_group('Web submission', 'Authenticate with the web submission endpoint')
+    ag_sendh.add_argument('--web-auth-new', dest='auth_new', action='store_true', default=False,
+                          help='Initiate a new web authentication request')
+    ag_sendh.add_argument('--web-auth-verify', dest='auth_verify', metavar='VERIFY_TOKEN',
+                          help='Submit the token received via verification email')
+    sp_send.set_defaults(func=cmd_send)
+
     cmdargs = parser.parse_args()
 
     logger.setLevel(logging.DEBUG)
